/*====================================+=====================================+
! File CServerList.h                  ! Copyright (C) 2018-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/dataview.h>
#include <wx/timer.h>
#include <wx/imaglist.h>
#include <wx/listctrl.h>
#include <wx/notebook.h>
#include "common/sc_base.h"
#include "CFrame.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CServer ;
using t_vec_server = std::vector< CServer > ;

/*-------------------------------------------------------------------------*/
class CServer
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      //CServer()
      //~CServer() ;

      /*--( Availability )-------------------------------------------------*/
      enum e_avail { AVA_UNKNOWN, AVA_OK, AVA_KO, AVA_NB } ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      bool  m_boo_select = false     ;
      /*-------------------------------------------------------------------*/
      wxString m_s_name              ;
      wxString m_s_ref               ;
      int      m_i_img_dc = -1       ;
      wxString m_s_dc                ;
      wxString m_s_price             ;
      wxString m_s_cpu               ;
      wxString m_s_memory            ;
      wxString m_s_storage           ;
      wxString m_s_network           ;
      wxString m_s_order_url         ;
      wxString m_s_detail_url        ;
      e_avail  m_avail = AVA_UNKNOWN ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void restore_data( const t_vec_server &vec_server_save ) ;
      wxString get_avail_url() const ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CServerList : public wxListCtrl
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CServerList( wxWindow *parent,
                   e_server_type sty, const char *p_c_main_url
                 ) ;
      //~CServerList()

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      enum e_col { COL_NAME, COL_REF,    COL_DC,      COL_PRICE,
                   COL_CPU,  COL_MEMORY, COL_STORAGE, COL_NETWORK,
                   COL_NB
                 } ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      e_server_type m_sty        ;
      wxString      m_s_main_url ;
      wxString      m_s_name     ;
      int           m_i_ind_ico  ;
      t_vec_server  m_vec_server ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxItemAttr *OnGetItemAttr( long l_item ) const override ;
      virtual int OnGetItemColumnImage( long l_item, long l_column
                                      ) const override ;
      virtual wxString OnGetItemText( long l_item, long l_column
                                    ) const override ;
      virtual bool OnGetItemIsChecked( long l_item ) const override ;
      void OnKeyDown( wxKeyEvent &event ) ;
      /*-------------------------------------------------------------------*/
      long get_next_item_hil( long l_item_from ) const
      {  return GetNextItem( l_item_from,
                              wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED
                            ) ;
      }
      long get_item_focus() const
      {  return GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED ) ; }
      void refresh_all_items() ;
      /*-------------------------------------------------------------------*/
      const char *get_name() const
      {  return wxGetApp().get_sty_name( m_sty ) ; }
      int set_select( long l_item, bool boo_select ) ;
      void select_all( bool boo_select ) ;
      void select_invert() ;
      void select_unselect_hil_following_focus() ;
      const CServer *get_server( long l_item ) const ;
      CServer *get_server( long l_item ) ;
      CServer *find_server( const wxString &s_ref ) ;
      wxString get_order_url( long l_item ) const ;
      wxString get_detail_url( long l_item ) const ;
      wxString get_main_url() const ;
      /*-------------------------------------------------------------------*/
      int launch_refresh() ;
      virtual int refresh( int i_err_code, const wxString &co_s_data ) = 0 ;
      int pre_refresh( int i_err_code, const wxString &co_s_data ) ;
      int post_refresh( int i_ret_pre ) ;
      int check_avail( int i_err_code,
                       const wxString &co_s_ref,
                       const wxString &co_s_data
                     ) ;
      /*-------------------------------------------------------------------*/
      void context_menu() ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CSourceKimsufi : public CServerList
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CSourceKimsufi( wxWindow *parent )
                    : CServerList( parent,
                                   STY_KIMSUFI, "https://www.kimsufi.com"
                                 )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual int refresh( int i_err_code, const wxString &co_s_data
                         ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CSourceSoYouStart : public CServerList
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CSourceSoYouStart( wxWindow *parent, e_server_type sty )
                       : CServerList( parent,
                                      sty, "https://www.soyoustart.com"
                                    )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void add_one_server_per_dc( CServer &server,
                                  const wxString &s_dc,
                                  const t_vec_server &vec_server_save
                                ) ;
      void refresh_deals( const wxString &co_s_data,
                          const t_vec_server &vec_server_save
                        ) ;
      void refresh_standard( const wxString &co_s_data,
                             const t_vec_server &vec_server_save
                           ) ;
      virtual int refresh( int i_err_code, const wxString &co_s_data
                         ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CNotebook : public wxNotebook
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CNotebook( wxWindow *parent ) ;
      ~CNotebook() ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxTimer m_timer_check_avail ;
      /*--( References to select after refresh )---------------------------*/
      sc::t_vec_str m_vec_ref_to_select ;
      /*-------------------------------------------------------------------*/
      wxImageList *m_il = nullptr ;
      int m_i_img_flag_canada         = -1 ;
      int m_i_img_flag_france         = -1 ;
      int m_i_img_flag_germany        = -1 ;
      int m_i_img_flag_poland         = -1 ;
      int m_i_img_flag_united_kingdom = -1 ;
      int m_a_i_img_source[ STY_NB ] ;
      /*-------------------------------------------------------------------*/
      const wxString m_co_s_col_info_sep = " - " ;
      /*--( Number of global server refresh and availabilities started )---*/
      int m_i_nb_refresh_server = 0 ;
      int m_i_nb_check_avail    = 0 ;
      /*--( How many server sources have been refreshed )------------------*/
      int m_i_source_refresh_done ;
      /*--( How many server availabilities have to be/been refreshed )-----*/
      int m_i_avail_refresh_todo ;
      int m_i_avail_refresh_done ;
      /*--( page/item to show after a "check_avail" with positive result )-*/
      int  m_i_page_to_show_avail_ok ;
      long m_l_item_to_show_avail_ok ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void add_page_server_list( CServerList *server_list ) ;
      void add_pages() ;
      CServerList *get_server_list( int i_sty ) const
      {  return static_cast< CServerList * >( GetPage( i_sty ) ) ; }
      CServerList *get_current_server_list() const
      {  return static_cast< CServerList * >( GetPage( GetSelection() ) ) ; }
      /*-------------------------------------------------------------------*/
      int set_select( const wxString &s_ref, bool boo_select ) ;
      void select_ref_to_select() ;
      void string_to_col_width( const wxString &s_val ) ;
      wxString col_width_to_string() const ;
      void string_to_serv_ref_sel( const wxString &s_val ) ;
      wxString serv_ref_sel_to_string() const ;
      /*-------------------------------------------------------------------*/
      void launch_refresh_server() ;
      int launch_check_avail( bool boo_only_current_page = false ) ;
      /*-------------------------------------------------------------------*/
      void show_info_after_avail_ok() ;
      bool get_auto_check_avail_running() const
      {  return m_timer_check_avail.IsRunning() ; }
      int start_auto_check_avail() ;
      int stop_auto_check_avail() ;
      /*-------------------------------------------------------------------*/
      void context_menu() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CServerList.h                          !
+==========================================================================*/
