/*====================================+=====================================+
! File app_info.h                     ! Copyright (C) 2018-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+---------+-----------------------------------------------------------------+
! History !                                                                 !
+---------+                                                                 !
! Date        Version  Description                                          !
! ----------  -------  ---------------------------------------------------- !
! 2020/11/29  1.15     released (build 156) (SYS parsing)                   !
! 2020/10/25  1.14     released (build 153) (SYS parsing)                   !
! 2020/10/11  1.13     released (build 151) (SYS parsing)                   !
! 2020/09/20  1.12     released (build 149) (-Storage,-Maxcores, parsing)   !
! 2020/01/05  1.11     released (build 129)                                 !
! 2019/12/01  1.10     released (build 126) (SoYouStart)                    !
! 2019/07/28  1.00     released (build 086)                                 !
!                                                                           !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include "app_build_number.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
#define APP_NAME                "kimfo"
#define APP_DISPLAY_NAME_ASCII  "Kimfo"
#define APP_DISPLAY_NAME_UTF8   APP_DISPLAY_NAME_ASCII
#define APP_DISPLAY_NAME        APP_DISPLAY_NAME_ASCII
#define APP_DISPLAY_NAME_8859   APP_DISPLAY_NAME_ASCII
/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define APP_HOST          "scarabee-software.net"
#define APP_URL_SITE      APP_HOST
#define APP_URL_FORUM     APP_HOST "/forum"
#define APP_URL_DONATION  "https://www.paypal.com/cgi-bin/webscr?" \
                          "cmd=_s-xclick&hosted_button_id=BHASPCZZFZU62"
#define APP_URL_VERSION \
        "http://" APP_HOST "/download/kimfo/kimfo_version.txt"
/*-------------------------------------------------------------------------*/
#ifdef __x86_64__
   #define APP_CPU_TYPE     "x86_64"
   #define APP_BITNESS_NUM  64
   #define APP_BITNESS_TEXT wxTRANSLATE( "64-bit" )
#elif defined( __i386__ )
   #define APP_CPU_TYPE     "i386"
   #define APP_BITNESS_NUM  32
   #define APP_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#elif defined( __arm__ )
   #define APP_CPU_TYPE     "armhf"
   #define APP_BITNESS_NUM  32
   #define APP_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#else
   #error Unknown platform to: define cpu type and bitness
#endif
/*-------------------------------------------------------------------------*/
#ifdef __WXMSW__
   #define APP_OS_TYPE "windows"
#elif defined( __WXGTK__ )
   #define APP_OS_TYPE "gnulinux"
#else
   #error Unknown platform to: define APP_OS_TYPE
#endif
/*-------------------------------------------------------------------------*/
#define APP_BUILT_TYPE  APP_OS_TYPE "_" APP_CPU_TYPE
/*-------------------------------------------------------------------------*/
#define APP_VERSION       "1.15" // " alpha 1 (" APP_BUILD_NUMBER ")"
#define APP_VERSION_MSWRC  1,15,0,0
/*-------------------------------------------------------------------------*/
#define APP_DATES          "2018-2020"
#define APP_COMPANY_ASCII  "Remi PASCAL"
#define APP_COMPANY_UTF8   u8"R\u00e9mi PASCAL"
#define APP_COMPANY        wxString::FromUTF8( APP_COMPANY_UTF8 )
#define APP_COMPANY_8859   "R\xe9mi PASCAL"
/*-------------------------------------------------------------------------*/
#define APP_COPYRIGHT_ASCII \
        "Copyright (C) " APP_DATES " " APP_COMPANY_ASCII
#define APP_COPYRIGHT_8859 \
        "Copyright \xa9 " APP_DATES " " APP_COMPANY_8859
#define APP_COPYRIGHT \
          wxString::FromUTF8( u8"Copyright \u00a9" ) \
        + " " APP_DATES " " + APP_COMPANY
/*-------------------------------------------------------------------------*/
#define APP_DESCRIPTION_8859 \
        wxTRANSLATE( "Kimfo : check Kimsufi and SoYouStart " \
                     "servers availability" \
                   )
#define APP_DESCRIPTION APP_DESCRIPTION_8859
/*-------------------------------------------------------------------------*/
#define APP_LICENSE_ASCII \
        wxTRANSLATE( "Kimfo is free of charge and " \
                     "distributed under the GPLv3 license" \
                   )
#define APP_LICENSE APP_LICENSE_ASCII

/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define APP_LICENSE_URL "www.gnu.org/licenses/gpl.html"

/*-------------------------------------------------------------------------*/
#define APP_MAIL "kimfo <kimfo@scarabee-software.net>"
#define APP_MAIL_TRANSLATOR \
        "English <kimfo.translation@scarabee-software.net>"
/*--------------------------------------------------------------------------+
! IPC communication port for single instance management                     !
! It is only used for the gnu/linux version, on MSW it is DDE               !
+--------------------------------------------------------------------------*/
#define APP_SINS_SERVICE  "19222"
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                            End of file app_info.h                         !
+==========================================================================*/
