/*====================================+=====================================+
! File sc_base.h                      ! Copyright (C) 2018-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This is free software: you can redistribute it and/or modify it under the !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <vector>
#include <map>
#include <deque>
#include <list>
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/mstream.h>
#include <wx/artprov.h>
#include <wx/ipc.h>
/*-------------------------------------------------------------------------*/
class CConfigRect ;
/*-------------------------------------------------------------------------*/



/*--( Scarabee Common )----------------------------------------------------*/
namespace sc
{
/*-------------------------------------------------------------------------*/
using wxString_it      = wxString::iterator ;
using wxString_cit     = wxString::const_iterator ;
using wxString_crit    = wxString::const_reverse_iterator ;
/*-------------------------------------------------------------------------*/
using t_vec_str        = std::vector< wxString > ;
using t_vec_str_it     = t_vec_str::iterator ;
using t_vec_str_cit    = t_vec_str::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_vec_int        = std::vector< int > ;
using t_vec_int_it     = t_vec_int::iterator ;
using t_vec_int_cit    = t_vec_int::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_vec_boo        = std::vector< bool > ;
using t_vec_boo_it     = t_vec_boo::iterator ;
using t_vec_boo_cit    = t_vec_boo::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_map_strstr     = std::map< wxString, wxString > ;
using t_map_strstr_it  = t_map_strstr::iterator ;
using t_map_strstr_cit = t_map_strstr::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_map_strint     = std::map< wxString, int > ;
using t_map_strint_it  = t_map_strint::iterator ;
using t_map_strint_cit = t_map_strint::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_map_intstr     = std::map< int, wxString > ;
using t_map_intstr_it  = t_map_intstr::iterator ;
using t_map_intstr_cit = t_map_intstr::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_deq_string     = std::deque< wxString > ;
using t_deq_string_it  = t_deq_string::iterator ;
using t_deq_string_cit = t_deq_string::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_lst_string     = std::list< wxString > ;
using t_lst_string_it  = t_lst_string::iterator ;
using t_lst_string_cit = t_lst_string::const_iterator ;


/*-------------------------------------------------------------------------*/
int set_clipboard( const wxString &s ) ;
int get_clipboard( wxString &s ) ;
/*-------------------------------------------------------------------------*/
wxBitmap get_menu_bmp( const wxArtID &id ) ;
wxBitmap get_toolbar_bmp( const wxArtID &id ) ;
wxMenuItem *insert_item( wxMenu *menu, int i_pos, int i_id,
                         const wxString &s_item,
                         const wxBitmap &bmp /*= wxNullBitmap*/,
                         const wxString &s_help = wxEmptyString,
                         wxItemKind mik = wxITEM_NORMAL,
                         wxMenu *p_submenu = nullptr
                       ) ;
wxMenuItem *append_item( wxMenu *menu, int i_id,
                         const wxString &s_item,
                         const wxBitmap &bmp /*= wxNullBitmap*/,
                         const wxString &s_help = wxEmptyString,
                         wxItemKind mik = wxITEM_NORMAL,
                         wxMenu *p_submenu = nullptr
                       ) ;
wxMenuItem *append_item( wxMenu *menu, int i_id, const wxArtID &artid ) ;
wxMenuItem *append_item( wxMenu *dst, wxMenuItem *mi_src ) ;
void append_menu( wxMenu *dst, wxMenu *src ) ;
wxMenuItem *append_item( wxMenu *dst, wxMenu *src, int i_id ) ;
wxMenuItem *append_item( wxMenu *dst, wxMenuBar *src, int i_id ) ;
/*-------------------------------------------------------------------------*/
wxRect get_screen_area_under( const wxWindow *win ) ;
bool is_rec_window_ok( const wxRect &rec ) ;
void set_tlw_rect( wxWindow *p_win, const wxRect &rect ) ;
void init_tlw_rect( wxWindow *p_win, const CConfigRect &cfg_rect ) ;
void save_tlw_rect( wxWindow *p_win, CConfigRect &cfg_rect ) ;
void set_font_fix( wxWindow *p_win, bool boo_fix ) ;
/*-------------------------------------------------------------------------*/
wxString sec2hms( int i_sec ) ;
wxBitmap scale_down_or_center( const wxBitmap &bmp,
                               const wxSize &size_target,
                               bool boo_high_quality = true
                             ) ;
/*-------------------------------------------------------------------------*/
int match( const wxString &s_data, const wxString &s_re ... ) ;
wxColor alt_bg_color( wxColor color_bg ) ;
/*-------------------------------------------------------------------------*/
wxString get_debug_filename() ;
void trace( const wxString &s ) ;

/*-------------------------------------------------------------------------*/
class CDlgProgress : public wxDialog
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CDlgProgress( wxWindow *parent ) ;
      //~CDlgProgress();

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      wxLongLong   m_ll_ms_last_display = 0 ;
      /*-------------------------------------------------------------------*/
      wxBoxSizer   *m_bsz_gen ;
      wxStaticText *m_stx_msg ;
      wxGauge      *m_ga_progress ;
      wxButton     *m_bt_quit ;

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      virtual void OnBtCancel( wxCommandEvent& event ) { event.Skip() ; }
      /*-------------------------------------------------------------------*/
      bool check_progress_display()
      {
         /*--( Enough time flied ? )---------------------------------------*/
         wxLongLong m_ll_ms = wxGetUTCTimeMillis() ;
         if( m_ll_ms - m_ll_ms_last_display < 200 ) { return false ; }
         /*--( Yes, the timestamp is saved and agreement given )-----------*/
         m_ll_ms_last_display = m_ll_ms ;
         return true ;
         /*----------------------------------------------------------------*/
      }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CBoolInScope
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CBoolInScope( bool &boo_var ) : m_boo_var( boo_var )
      {  m_boo_var = true ; }
      ~CBoolInScope()
      {  m_boo_var = false ; }

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      bool &m_boo_var ;
      /*-------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CEvent : public wxEvent
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CEvent( wxEventType eventType, int i_id = 0 )
            : wxEvent( i_id, eventType )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxEvent *Clone() const override
      {  return new CEvent( *this ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CStringEvent : public wxEvent
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CStringEvent( wxEventType eventType,
                    const wxString &s = wxEmptyString,
                    int i_id = 0
                  )
                  : wxEvent( i_id, eventType ), m_s_string( s )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString m_s_string ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxEvent *Clone() const override
      {  return new CStringEvent( *this ) ; }
      /*-------------------------------------------------------------------*/
      wxString get_string() const { return m_s_string ; }
      void set_string( const wxString &s ) { m_s_string = s ; }

   /*----------------------------------------------------------------------*/
} ;

/*--( SI : Single Instance )-----------------------------------------------*/
class CSIConnection : public wxConnection
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool OnExec( const wxString &s_topic, const wxString &s_data );
   /*----------------------------------------------------------------------*/
} ;

/*--( SI : Single Instance )-----------------------------------------------*/
class CSIServer : public wxServer
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxConnectionBase *OnAcceptConnection( const wxString &s_topic);
   /*----------------------------------------------------------------------*/
} ;

/*--( SI : Single Instance )-----------------------------------------------*/
class CSIClient : public wxClient
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxConnectionBase *OnMakeConnection()
      {  return new CSIConnection() ; }
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CStaticTextEllipsizeMiddle : public wxStaticText
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CStaticTextEllipsizeMiddle( wxWindow *parent,
                                  wxWindowID id,
                                  const wxString &label,
                                  const wxPoint &pos = wxDefaultPosition,
                                  const wxSize &size = wxDefaultSize,
                                  long style = 0,
                                  const wxString &name = wxStaticTextNameStr
                                )
                              : wxStaticText( parent, id, label, pos, size,
                                                style
                                              | wxST_ELLIPSIZE_MIDDLE
                                              | wxST_NO_AUTORESIZE,
                                              name
                                            )
      {  ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CStaticTextEllipsizeEnd : public wxStaticText
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CStaticTextEllipsizeEnd( wxWindow *parent,
                               wxWindowID id,
                               const wxString &label,
                               const wxPoint &pos = wxDefaultPosition,
                               const wxSize &size = wxDefaultSize,
                               long style = 0,
                               const wxString &name = wxStaticTextNameStr
                             )
                             : wxStaticText( parent, id, label, pos, size,
                                               style
                                             | wxST_ELLIPSIZE_END
                                             | wxST_NO_AUTORESIZE,
                                             name
                                           )
      {  ; }
   /*----------------------------------------------------------------------*/
} ;



/*-------------------------------------------------------------------------*/
} ; // namespace sc
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file sc_base.h                            !
+==========================================================================*/
